@@ -12,19 +12,19 @@ import trexmodel.conf as model_conf
 from trexlib.utils.security_util import generate_user_id, hash_password
 from trexlib.utils.string_util import random_number, is_empty, is_not_empty,\
     split_by_length, random_string
-import logging, hashlib
+import logging
 from datetime import datetime, timedelta
-from trexlib.utils.common.date_util import parse_datetime
 from trexmodel import conf, program_conf
-from google.auth._default import default
 from trexmodel.models.datastore.system_models import Tagging
 from trexmodel.models.datastore.model_decorators import model_transactional
+from trexlib.utils.common.common_util import sort_dict_list
 
 logger = logging.getLogger('model')
 #logger = logging.getLogger('debug')
 
 class BusinessEntity(BaseNModel, DictModel):
     company_name                = ndb.StringProperty(required=False)
+    brand_name                  = ndb.StringProperty(required=False)
     business_reg_no             = ndb.StringProperty(required=False)
     
 class MerchantMin(BusinessEntity, FullTextSearchable):
@@ -76,7 +76,7 @@ class MerchantAcct(MerchantMin):
     stat_figure_update_interval_in_minutes  = conf.MERCHANT_STAT_FIGURE_UPDATE_INTERVAL_IN_MINUTES
     stat_figure_update_datetime_format      = '%d-%m-%Y %H:%M:%S'
     
-    dict_properties = ['company_name', 'contact_name', 'business_reg_no', 'mobile_phone', 'office_phone', 'fax_phone', 'email', 'account_code', 'country',
+    dict_properties = ['company_name', 'brand_name', 'contact_name', 'business_reg_no', 'mobile_phone', 'office_phone', 'fax_phone', 'email', 'account_code', 'country',
                        'registered_datetime', 'modified_datetime', 'plan_start_date', 'plan_end_date', 'currency_code', 
                        'published_program_configuration', 'published_voucher_configuration', 'membership_configuration', 
                        'tier_membership_configuration', 'prepaid_configuration', 'product_modifier_configuration']
@@ -89,18 +89,14 @@ class MerchantAcct(MerchantMin):
                 'currency_code'     : self.currency_code,
                 'country'           : self.country,
                 }
-    @property
-    def account_settings(self):
-        return {
-                'account_code'  : self.account_code,
-                'currency'      : self.currency_code,
-                'locale'        : self.locale,    
-                }
+    
+       
     
     
     @staticmethod
-    def update_details(merchant_acct, company_name=None, business_reg_no=None, contact_name=None, email=None, mobile_phone=None, office_phone=None, currency_code=None, country=None):
+    def update_details(merchant_acct, company_name=None, brand_name=None, business_reg_no=None, contact_name=None, email=None, mobile_phone=None, office_phone=None, currency_code=None, country=None):
         merchant_acct.company_name      = company_name
+        merchant_acct.brand_name        = brand_name
         merchant_acct.business_reg_no   = business_reg_no
         merchant_acct.contact_name      = contact_name
         merchant_acct.email             = email
@@ -614,21 +610,24 @@ class MerchantAcct(MerchantMin):
         return None
     
     @staticmethod
-    def create(company_name=None, contact_name=None, email=None, mobile_phone=None, office_phone=None, plan_start_date=None, plan_end_date=None, 
-               account_code=None, currency_code=None):
+    def create(company_name=None, brand_name=None, contact_name=None, email=None, mobile_phone=None, office_phone=None, plan_start_date=None, plan_end_date=None, 
+               account_code=None, currency_code=None, country=None):
         
         if account_code is None:
             account_code    = "%s-%s-%s-%s" % (random_number(4),random_number(4),random_number(4),random_number(4))
             
         merchant_acct   = MerchantAcct(
-                                       company_name     = company_name, 
+                                       company_name     = company_name,
+                                       brand_name       = brand_name,     
                                        contact_name     = contact_name,
                                        email            = email,
                                        mobile_phone     = mobile_phone,
                                        office_phone     = office_phone,
                                        plan_start_date  = plan_start_date, 
                                        plan_end_date    = plan_end_date,
-                                       currency_code    = currency_code
+                                       currency_code    = currency_code,
+                                       country          = country,    
+                                       api_key          = random_string(24),
                                        )
         
         logging.debug('account_code=%s', account_code)
@@ -1192,4 +1191,59 @@ class ServiceTaxSetup(BaseNModel,DictModel):
     @staticmethod
     def list_by_merchant_acct(merchant_acct):
         return ServiceTaxSetup.query(ancestor = merchant_acct.create_ndb_key()).fetch(limit=conf.MAX_FETCH_RECORD)
+
+class BannerFile(BaseNModel, DictModel):
+    '''
+    Merchant Account as ancestor
+    '''
+    banner_file_type                = ndb.StringProperty(required=True)
+    banner_file_public_url          = ndb.StringProperty(required=True)
+    banner_file_storage_filename    = ndb.StringProperty(required=True)
+    sequence                        = ndb.IntegerProperty(required=False, default=0)
     
+    dict_properties = ['banner_file_public_url', 'banner_file_storage_filename', 'banner_file_type', 'sequence']
+    
+    @staticmethod
+    def list_by_merchant_acct(merchant_acct):
+        result = BannerFile.query(ancestor=merchant_acct.create_ndb_key()).fetch(limit=conf.MAX_FETCH_RECORD)
+        return result
+    
+    @staticmethod
+    def upload_file(uploading_file, merchant_acct, bucket, banner_file_type=None):
+        file_prefix                         = random_string(8)
+        banner_file_storage_filename       = 'merchant/'+merchant_acct.key_in_str+'/banner/'+file_prefix+'-'+uploading_file.filename
+        blob                                = bucket.blob(banner_file_storage_filename)
+        
+        logger.debug('banner_file_storage_filename=%s', banner_file_storage_filename)
+        
+        blob.upload_from_string(
+                uploading_file.read(),
+                content_type=uploading_file.content_type
+            )
+        
+        uploaded_url        = blob.public_url
+        
+        logger.debug('banner_file_type=%s', uploaded_url)
+        logger.debug('product_file_type=%s', banner_file_type)
+        
+        count = BannerFile.query(ancestor=merchant_acct.create_ndb_key()).count()
+        
+        banner_file = BannerFile(
+                            parent = merchant_acct.create_ndb_key(),
+                            banner_file_public_url              = uploaded_url,
+                            banner_file_storage_filename        = banner_file_storage_filename,
+                            banner_file_type                    = banner_file_type,
+                            sequence                            = count+1,
+                            )
+        
+        banner_file.put()
+        
+        return banner_file
+    
+    @staticmethod
+    def remove_file(banner_file, bucket):
+        
+        old_logo_blob = bucket.get_blob(banner_file.banner_file_storage_filename) 
+        if old_logo_blob:
+            old_logo_blob.delete()
+            banner_file.delete()    
