@@ -144,7 +144,8 @@ class DictModel(ndb.Model):
         logger.debug('DictModel: full=%s', full)
         logger.debug('DictModel: show_key=%s', show_key)
         
-        if self.show_key_in_dict or show_key:
+        #if self.show_key_in_dict or show_key:
+        if show_key:
             try:
                 result['key'] = self.key.urlsafe().decode("utf-8")
             except:
@@ -292,7 +293,7 @@ class BaseNModel(DictModel, NDBModel):
             (search_results, next_cursor, more)       = query.fetch_page(page_size=limit, start_cursor=start_cursor, keys_only=keys_only)
             logger.debug('##########################################')
             
-            logger.debug('list_all: search_results=%s', search_results)
+            logger.debug('list_all: results=%s', search_results)
             logger.debug('list_all: next_cursor=%s', next_cursor)
             logger.debug('list_all: more=%s', more) 
             
@@ -327,7 +328,11 @@ class FullTextSearchable(ndb.Model):
     
     def _pre_put_hook(self):
         """before save, parse searchable field into strings"""
+        
+        
+        
         _searchable_field_name = self.fulltextsearch_field_name
+        
         logger.debug('_searchable_field_name=%s', _searchable_field_name)
         if _searchable_field_name:
             searchable_field_value = getattr(self, _searchable_field_name)
