@@ -21,13 +21,14 @@ class POSSetting(BaseNModel,DictModel):
     '''
     pos_name                = ndb.StringProperty(required=True)
     activation_code         = ndb.StringProperty(required=True)
+    device_id               = ndb.StringProperty(required=False)
     activated               = ndb.BooleanProperty(required=True, default=False)
     assigned_outlet         = ndb.KeyProperty(name="assigned_outlet", kind=Outlet)
     #assigned_catalogue      = ndb.KeyProperty(name="assigned_outlet", kind=ProductCatalogue)
     created_datetime        = ndb.DateTimeProperty(required=True, auto_now_add=True)
     activated_datetime      = ndb.DateTimeProperty(required=False)
     
-    dict_properties = ['pos_name', 'activation_code', 'activated', 'assigned_outlet_key', 'activated_datetime', 'created_datetime']
+    dict_properties = ['pos_name', 'activation_code', 'device_id', 'activated', 'assigned_outlet_key', 'activated_datetime', 'created_datetime']
     
     @property
     def assigned_outlet_key(self):
@@ -69,10 +70,11 @@ class POSSetting(BaseNModel,DictModel):
         return pos_setting
     
     @staticmethod
-    def update(pos_setting_key, pos_name, assigned_outlet):
+    def update(pos_setting_key, pos_name, assigned_outlet, device_id):
         pos_setting                         = POSSetting.fetch(pos_setting_key)
         pos_setting.pos_name                = pos_name
         pos_setting.assigned_outlet         = assigned_outlet.create_ndb_key()
+        pos_setting.device_id               = device_id
         pos_setting.put()
         
         return pos_setting
@@ -113,16 +115,12 @@ class POSSetting(BaseNModel,DictModel):
         
         return POSSetting.count_with_condition_query(query)
         
-    @staticmethod
-    def activate(activation_code):
-        checking_pos_setting = POSSetting.get_by_activation_code(activation_code)
-        if checking_pos_setting:
-            checking_pos_setting.activated          = True
-            checking_pos_setting.activated_datetime = datetime.utcnow()
-            checking_pos_setting.put()
-            return True
-        else:
-            return False
+    def activate(self, device_id):
+        self.device_id          = device_id
+        #self.activated          = True
+        self.activated_datetime = datetime.utcnow()
+        self.put()
+        
         
     @staticmethod
     def remove_by_activation_code(activation_code):
@@ -132,7 +130,7 @@ class POSSetting(BaseNModel,DictModel):
             return True
         else:
             return False
-        
+    '''    
     @property
     def setting(self):
         merchant_acct           = MerchantAcct.fetch(self.parent_key)
@@ -149,7 +147,9 @@ class POSSetting(BaseNModel,DictModel):
                 dinning_option_json.append({
                                             'option_key'                : d.key_in_str,
                                             'option_name'               : d.name,
+                                            'option_prefix'             : d.prefix,
                                             'is_default'                : d.is_default,
+                                            'is_dinning_input'          : d.is_dinning_input,
                                             'dinning_table_is_required' : d.dinning_table_is_required,
                                             'assign_queue'              : d.assign_queue,
                                             })
@@ -198,7 +198,8 @@ class POSSetting(BaseNModel,DictModel):
                         'address'                    : outlet.address,
                         'email'                      : outlet.email,
                         'phone'                      : outlet.office_phone,
-                        'website'                    : merchant_acct.website or '',   
+                        'website'                    : merchant_acct.website or '',  
+                        'outlet_name'                : outlet.name,    
                         }
         
         return {
@@ -206,21 +207,13 @@ class POSSetting(BaseNModel,DictModel):
                 'pos_name'                          : self.pos_name,
                 'company_name'                      : merchant_acct.company_name,
                 'website'                           : merchant_acct.website,
-                #'outlet_key'                        : self.assigned_outlet_key,
-                #'outlet_id'                         : outlet.id,    
-                #'outlet_company_name'               : outlet.company_name,
-                #'outlet_business_reg_no'            : outlet.business_reg_no,
-                #'outlet_address'                    : outlet.address,
-                #'outlet_email'                      : outlet.email,
-                #'outlet_phone'                      : outlet.office_phone,
-                'website'                           : merchant_acct.website,
                 'account_id'                        : self.parent_key,
                 'api_key'                           : merchant_acct.api_key,
                 'logo_image_url'                    : merchant_acct.logo_public_url,
                 'account_settings'                  : account_settings,
                 'outlet_details'                    : outlet_details,
                 }
-        
+    '''    
         
 class POSCatalogue(BaseNModel,DictModel):
     '''
@@ -366,23 +359,49 @@ class DinningTableSetup(BaseNModel,DictModel):
     @staticmethod
     def list_by_merchant_acct(merchant_acct):
         return DinningTableSetup.query(ancestor = merchant_acct.create_ndb_key()).fetch(limit=conf.MAX_FETCH_RECORD)
+    
+    def generate_weblink_details_list(self, outlet_key, dinning_option, weblink_url_format):
+        table_list_in_list = self.table_list.split(',')
+        table_weblink_details_list = []
+        for table in table_list_in_list:
+            web_link = weblink_url_format.format(outlet_key=outlet_key, dinning_option=dinning_option, table=table)
+            table_weblink_details_list.append({
+                                            'table'             : table,
+                                            'dinning_option'    : dinning_option,
+                                            'web_link'          :  web_link,  
+                                             
+                                            })
+        
+        return table_weblink_details_list
         
 class DinningOption(BaseNModel,DictModel):
-    name                        = ndb.StringProperty(required=True)  
+    name                        = ndb.StringProperty(required=True)
+    prefix                      = ndb.StringProperty(required=False)  
     is_default                  = ndb.BooleanProperty(default=False)
+    is_dinning_input            = ndb.BooleanProperty(default=True)
+    is_takeaway_input           = ndb.BooleanProperty(default=False)
+    is_delivery_input           = ndb.BooleanProperty(default=False)
+    is_self_order_input         = ndb.BooleanProperty(default=False)
+    is_self_payment_mandatory   = ndb.BooleanProperty(default=False)
     is_archive                  = ndb.BooleanProperty(default=False)
     dinning_table_is_required   = ndb.BooleanProperty(default=False)
     assign_queue                = ndb.BooleanProperty(default=False) 
     modified_datetime           = ndb.DateTimeProperty(auto_now=True) 
     archived_datetime           = ndb.DateTimeProperty(required=False) 
     
-    dict_properties = ['name','is_archive', 'is_default', 'dinning_table_is_required', 'assign_queue']
+    dict_properties = ['name','prefix', 'is_dinning_input', 'is_delivery_input', 'is_takeaway_input', 'is_self_order_input', 'is_self_payment_mandatory', 'is_archive', 'is_default', 'dinning_table_is_required', 'assign_queue']
     
     @staticmethod
-    def create(name, merchant_acct, is_default=False, dinning_table_is_required=False, assign_queue=False):
+    def create(name, prefix, merchant_acct, is_dinning_input=False, is_delivery_input=False, is_takeaway_input=False, is_self_order_input=False, is_self_payment_mandatory=False, is_default=False, dinning_table_is_required=False, assign_queue=False):
         dinning_option = DinningOption(
                                         parent                      = merchant_acct.create_ndb_key(),
                                         name                        = name,
+                                        prefix                      = prefix,
+                                        is_dinning_input            = is_dinning_input, 
+                                        is_delivery_input           = is_delivery_input,   
+                                        is_takeaway_input           = is_takeaway_input,
+                                        is_self_order_input         = is_self_order_input,
+                                        is_self_payment_mandatory   = is_self_payment_mandatory,    
                                         is_default                  = is_default,
                                         dinning_table_is_required   = dinning_table_is_required,
                                         assign_queue                = assign_queue,
@@ -391,9 +410,15 @@ class DinningOption(BaseNModel,DictModel):
         return dinning_option
     
     @staticmethod
-    def update(dinning_option, name, is_default=False, dinning_table_is_required=False, assign_queue=False):
+    def update(dinning_option, name, prefix, is_dinning_input=False, is_delivery_input=False, is_takeaway_input=False, is_self_order_input=False, is_self_payment_mandatory=False, is_default=False, dinning_table_is_required=False, assign_queue=False):
         dinning_option.name                         = name
+        dinning_option.prefix                       = prefix
         dinning_option.is_default                   = is_default
+        dinning_option.is_dinning_input             = is_dinning_input
+        dinning_option.is_delivery_input            = is_delivery_input
+        dinning_option.is_takeaway_input            = is_takeaway_input
+        dinning_option.is_self_order_input          = is_self_order_input
+        dinning_option.is_self_payment_mandatory    = is_self_payment_mandatory
         dinning_option.dinning_table_is_required    = dinning_table_is_required
         dinning_option.assign_queue                 = assign_queue
         dinning_option.put()
