@@ -26,6 +26,31 @@ class ProductBase(BaseNModel,DictModel):
     @property
     def merchant_acct_entity(self):
         return MerchantAcct.fetch(self.key.parent().urlsafe())
+'''
+class ProductCategorySetup(ProductBase):
+    
+    setup_label             = ndb.StringProperty(required=True)
+    created_datetime        = ndb.DateTimeProperty(required=True, auto_now_add=True)
+    modified_datetime       = ndb.DateTimeProperty(required=True, auto_now=True)
+    created_by              = ndb.KeyProperty(name="created_by", kind=MerchantUser)
+    modified_by             = ndb.KeyProperty(name="modified_by", kind=MerchantUser)
+    
+    dict_properties = ['setup_label', 'created_datetime', 'modified_datetime']
+    
+    @staticmethod
+    @model_transactional(desc='ProductCategorySetup.create')
+    def create(setup_label, merchant_acct, created_by=None):
+        created_by_key = None
+        if created_by:
+            created_by_key = created_by.create_ndb_key()
+        created_category_setup = ProductCategorySetup(
+                                            parent               = merchant_acct.create_ndb_key(),
+                                            setup_label          = setup_label,
+                                            created_by           = created_by_key,
+                                            )
+        
+        created_category_setup.put()
+'''    
 
 class ProductCategory(ProductBase):
     '''
@@ -43,6 +68,8 @@ class ProductCategory(ProductBase):
     created_by              = ndb.KeyProperty(name="created_by", kind=MerchantUser)
     modified_by             = ndb.KeyProperty(name="modified_by", kind=MerchantUser)
     
+    #category_setup          = ndb.KeyProperty(name="category_setup", kind=ProductCategorySetup)
+    
     
     dict_properties = ['category_label', 'parent_category_key', 'is_main_category', 'has_child', 'child_category_keys', 'product_item_count', 'product_modifier',
                        'product_items', 'category_label_and_other_details', 'category_label_and_product_name']
@@ -76,14 +103,6 @@ class ProductCategory(ProductBase):
                 child_category_dict_list.append(c)
         
         return child_category_dict_list
-    '''
-    @property
-    def category_label_and_other_details(self):
-        if self.has_child==False:
-            return '%s (%s %s)' % (self.category_label, self.product_item_count, gettext('items')) 
-        else:
-            return self.category_label
-    '''
         
     @property
     def category_label_and_other_details(self):
@@ -399,19 +418,20 @@ class Product(ProductBase, FullTextSearchable):
         modifier_setting_list_for_pos   = []
         
         
+        if modifier_keys_list:
+            product_modifiers_list = ProductModifier.list_by_merchant_account(merchant_acct)
         
-        product_modifiers_list = ProductModifier.list_by_merchant_account(merchant_acct)
-        
-        if product_modifiers_list:
-            for modifier_key in modifier_keys_list:
-                for modifier_details in product_modifiers_list:
-                    if modifier_details.key_in_str == modifier_key:
-                        modifier_setting_list_for_pos.append(modifier_details.setting_details_for_pos())
-                        break  
+            if product_modifiers_list:
+                for modifier_key in modifier_keys_list:
+                    for modifier_details in product_modifiers_list:
+                        if modifier_details.key_in_str == modifier_key:
+                            modifier_setting_list_for_pos.append(modifier_details.setting_details_for_pos())
+                            break  
         
         details_in_dict = {
                             'sku'                   : self.product_sku,
                             'label'                 : self.product_name,
+                            'desc'                  : self.product_desc,
                             'price'                 : self.price,
                             'barcode'               : self.barcode,   
                             'modifiers'             : modifier_setting_list_for_pos, 
@@ -460,7 +480,7 @@ class Product(ProductBase, FullTextSearchable):
         pos_settings        = {
                                     'represenation_sku'             : product_sku,
                                     'represenation_name'            : product_name,
-                                    'product_shortcut_key'          : product_sku,
+                                    'product_shortcut_key'          : '',
                                     }
         
         created_product = Product(
@@ -625,6 +645,7 @@ class ProductModifier(ProductBase):
     merchant_acct as ancestor
     '''
     modifier_name               = ndb.StringProperty(required=True)
+    modifier_label              = ndb.StringProperty(required=False)
     have_default_option         = ndb.BooleanProperty(required=True, default=False)
     modifier_options            = ndb.JsonProperty()
     
@@ -636,7 +657,7 @@ class ProductModifier(ProductBase):
     created_datetime            = ndb.DateTimeProperty(required=True, auto_now_add=True)
     modified_datetime           = ndb.DateTimeProperty(required=True, auto_now=True)
     
-    dict_properties = ['modifier_name', 'modifier_options', 'have_default_option', 'allow_multiple_option', 'option_is_mandatory', 'enabled', 'archived']
+    dict_properties = ['modifier_name', 'modifier_label', 'modifier_options', 'have_default_option', 'allow_multiple_option', 'option_is_mandatory', 'enabled', 'archived']
     
     @property
     def is_enabled(self):
@@ -654,6 +675,7 @@ class ProductModifier(ProductBase):
         return {
                 'modifier_key'          : self.key_in_str,
                 'modifier_name'         : self.modifier_name,
+                'modifier_label'        : self.modifier_label,
                 'have_default_option'   : self.have_default_option,
                 'allow_multiple_option' : self.allow_multiple_option,
                 'option_is_mandatory'   : self.option_is_mandatory,
@@ -663,10 +685,11 @@ class ProductModifier(ProductBase):
     
     
     @staticmethod
-    def create(merchant_acct, modifier_name, modifier_options={}, have_default_option=False, allow_multiple_option=False, option_is_mandatory=False):
+    def create(merchant_acct, modifier_name, modifier_label=None, modifier_options={}, have_default_option=False, allow_multiple_option=False, option_is_mandatory=False):
         product_modifier = ProductModifier(
                                     parent                  = merchant_acct.create_ndb_key(),
                                     modifier_name           = modifier_name,
+                                    modifier_label          = modifier_label,
                                     have_default_option     = have_default_option,
                                     allow_multiple_option   = allow_multiple_option,
                                     modifier_options        = modifier_options,
@@ -677,8 +700,9 @@ class ProductModifier(ProductBase):
         return product_modifier
     
     @staticmethod
-    def update(product_modifier, modifier_name, modifier_options={}, have_default_option=False, allow_multiple_option=False, option_is_mandatory=False):
+    def update(product_modifier, modifier_name, modifier_label=None, modifier_options={}, have_default_option=False, allow_multiple_option=False, option_is_mandatory=False):
         product_modifier.modifier_name              = modifier_name
+        product_modifier.modifier_label             = modifier_label
         product_modifier.modifier_options           = modifier_options
         product_modifier.have_default_option        = have_default_option
         product_modifier.allow_multiple_option      = allow_multiple_option
