@@ -6,7 +6,7 @@ Created on 19 Feb 2021
 from google.cloud import ndb
 from trexmodel.models.datastore.ndb_models import BaseNModel, DictModel
 import trexmodel.conf as model_conf
-from trexlib.utils.string_util import is_not_empty
+from trexlib.utils.string_util import is_not_empty, random_string
 from trexmodel.models.datastore.merchant_models import MerchantAcct, Outlet,\
     MerchantUser
 from trexmodel.models.datastore.coporate_models import CorporateAcct
@@ -15,16 +15,18 @@ from trexlib.utils.common.cache_util import cache
 from trexmodel import conf, program_conf
 from datetime import datetime
 from trexmodel.models.datastore.model_decorators import model_transactional
+from trexlib.utils.common.common_util import sort_list, sort_dict_list
+    
 
 
-logger = logging.getLogger('model')
+#logger = logging.getLogger('model')
+logger = logging.getLogger('debug')
 
-
-class ProgramBase(BaseNModel, DictModel):
+class BaseProgram(BaseNModel, DictModel):
     reward_base                         = ndb.StringProperty(required=True, choices=set(program_conf.REWARD_BASE_SET))
     reward_format                       = ndb.StringProperty(required=False, choices=set(program_conf.REWARD_FORMAT_SET))
-    completed_status                    = ndb.StringProperty(required=True, choices=set(program_conf.PROGRAM_STATUS))
-    giveaway_method                     = ndb.StringProperty(required=False, default=program_conf.PROGRAM_REWARD_GIVEAWAY_METHOD_SYSTEM)
+    completed_status                    = ndb.StringProperty(required=True, choices=set(program_conf.ALL_PROGRAM_STATUS))
+    giveaway_method                     = ndb.StringProperty(required=True, default=program_conf.PROGRAM_REWARD_GIVEAWAY_METHOD_SYSTEM)
     start_date                          = ndb.DateProperty(required=True)
     end_date                            = ndb.DateProperty(required=True)
     label                               = ndb.StringProperty(required=False)
@@ -65,12 +67,19 @@ class ProgramBase(BaseNModel, DictModel):
         return program_conf.is_existing_program_status_final_state(self.completed_status)
     
     @property
-    def completed_prograss_in_percentage(self):
-        return program_conf.program_completed_progress_percentage(self.completed_status)
-    
+    def exclusive_tags_list(self):
+        if self.program_settings.get('exclusivity') and self.program_settings.get('exclusivity').get('tags'):
+            return ','.join(self.program_settings.get('exclusivity').get('tags')) or ''
+        
     @property
-    def completed_status_index(self):
-        return program_conf.get_program_completed_status_index(self.completed_status)
+    def exclusive_memberships_list(self):
+        if self.program_settings.get('exclusivity') and self.program_settings.get('exclusivity').get('memberships'):
+            return ','.join(self.program_settings.get('exclusivity').get('memberships')) or ''
+        
+    @property
+    def exclusive_tier_memberships_list(self):
+        if self.program_settings.get('exclusivity') and self.program_settings.get('exclusivity').get('tier_memberships'):
+            return ','.join(self.program_settings.get('exclusivity').get('tier_memberships')) or ''  
     
     @property
     def spending_currency(self):
@@ -81,18 +90,19 @@ class ProgramBase(BaseNModel, DictModel):
         return self.program_settings.get('scheme').get('reward_amount')
     
     @property
-    def is_reward_amount_required(self):
-        return self.reward_base!=program_conf.REWARD_FORMAT_VOUCHER
-    
-    @property
-    def is_multitier_scheme(self):
+    def reward_limit_type(self):
         if self.program_settings and self.program_settings.get('scheme'):
-            return self.program_settings.get('scheme').get('is_multitier_scheme') or False
+            return self.program_settings.get('scheme').get('reward_limit_type')
+        
+    @property
+    def reward_limit_amount(self):
+        if self.program_settings and self.program_settings.get('scheme'):
+            return self.program_settings.get('scheme').get('reward_limit_amount')
     
     @property
-    def is_multitier_membership(self):
-        return self.program_settings and self.program_settings.get('is_multitier_membership')
-        
+    def is_reward_amount_required(self):
+        return self.reward_format in (program_conf.REWARD_FORMAT_POINT, program_conf.REWARD_FORMAT_STAMP)
+    
     @property
     def is_recurring_scheme(self):
         if self.program_settings and self.program_settings.get('scheme'):
@@ -118,22 +128,6 @@ class ProgramBase(BaseNModel, DictModel):
         if self.program_settings and self.program_settings.get('scheme'):
             return self.program_settings.get('scheme').get('expiration_value')                
     
-    
-    @property
-    def exclusive_tags_list(self):
-        if self.program_settings.get('exclusivity') and self.program_settings.get('exclusivity').get('tags'):
-            return ','.join(self.program_settings.get('exclusivity').get('tags')) or ''
-        
-    @property
-    def exclusive_memberships_list(self):
-        if self.program_settings.get('exclusivity') and self.program_settings.get('exclusivity').get('memberships'):
-            return ','.join(self.program_settings.get('exclusivity').get('memberships')) or ''
-        
-    @property
-    def exclusive_tier_memberships_list(self):
-        if self.program_settings.get('exclusivity') and self.program_settings.get('exclusivity').get('tier_memberships'):
-            return ','.join(self.program_settings.get('exclusivity').get('tier_memberships')) or ''        
-    
     @property
     def giveaway_reward_when(self):
         if self.program_settings.get('scheme') and self.program_settings.get('scheme').get('giveaway_when'):
@@ -144,8 +138,111 @@ class ProgramBase(BaseNModel, DictModel):
         if self.program_settings.get('scheme') and self.program_settings.get('scheme').get('advance_in_day'):
             return self.program_settings.get('scheme').get('advance_in_day')
         else:
-            return 0    
+            return 0 
     
+    @classmethod
+    def update_prorgram_exclusivity_data(cls, program, exclusivity_configuration={}, modified_by=None):
+        modified_by_username = None
+        
+        if is_not_empty(modified_by):
+            if isinstance(modified_by, MerchantUser):
+                modified_by_username = modified_by.username
+        
+        program_settings = program.program_settings
+        program_settings['exclusivity'] = exclusivity_configuration
+            
+        program.modified_by            = modified_by.create_ndb_key()
+        program.modified_by_username   = modified_by_username
+        program.completed_status       = program_conf.PROGRAM_STATUS_REWARD_EXCLUSIVITY
+        program.program_settings       = program_settings
+        
+        program.put()
+        
+        return program
+    
+    @classmethod
+    def __create_schedule_program(cls, merchant_acct, program):
+        if program.reward_base in (program_conf.SCHEDULE_BASED_PROGRAM):
+            if program.reward_base == program_conf.REWARD_BASE_ON_BIRTHDAY:
+                if program.giveaway_reward_when == program_conf.PROGRAM_SCHEDULE_TYPE_MONTH_START:
+                    MerchantScheduleProgram.create_first_day_of_month_schedule_program(merchant_acct, program)
+                elif program.giveaway_reward_when == program_conf.ADVANCE_IN_DAY:
+                    MerchantScheduleProgram.create_daily_schedule_program(merchant_acct, program)
+    
+    @classmethod
+    def __remove_schedule_program(cls, program):
+        if program.reward_base in (program_conf.SCHEDULE_BASED_PROGRAM):
+            MerchantScheduleProgram.remove_by_merchant_program(program)
+        
+    @classmethod
+    @model_transactional(desc="publish_program")
+    def publish_program(cls, program):
+        program.completed_status = program_conf.PROGRAM_STATUS_PUBLISH
+        program.published_datetime = datetime.now()
+        program.put()
+        
+        merchant_acct = program.merchant_acct
+        merchant_acct.update_published_program(program.to_program_configuration())
+        
+        cls.__create_schedule_program(merchant_acct, program)
+        
+    @classmethod
+    @model_transactional(desc="archive_program")
+    def archive_program(cls, program):
+        program.archived = True
+        program.archived_datetime = datetime.now()
+        program.put()
+        
+        merchant_acct = program.merchant_acct
+        merchant_acct.remove_program_from_published_program_configuration(program.key_in_str) 
+        
+        cls.__remove_schedule_program(program)
+        
+    
+    @classmethod
+    @model_transactional(desc="enable program")
+    def enable(cls, program, modified_by=None):
+        program.enabled = True
+        
+        modified_by_username = None
+        
+        if is_not_empty(modified_by):
+            if isinstance(modified_by, MerchantUser):
+                modified_by_username = modified_by.username
+        
+        program.modified_by            = modified_by.create_ndb_key()
+        program.modified_by_username   = modified_by_username
+        
+        program.put()
+        
+        if program.is_published:
+            merchant_acct = program.merchant_acct
+            merchant_acct.update_published_program(program.to_program_configuration())
+            
+            cls.__create_schedule_program(merchant_acct, program)
+            
+        
+    @classmethod
+    @model_transactional(desc="disable program")
+    def disable(cls, program, modified_by=None):
+        program.enabled = False
+        
+        modified_by_username = None
+        
+        if is_not_empty(modified_by):
+            if isinstance(modified_by, MerchantUser):
+                modified_by_username = modified_by.username
+                program.modified_by            = modified_by.create_ndb_key()
+                
+        program.modified_by_username   = modified_by_username
+        
+        program.put()  
+        
+        merchant_acct = program.merchant_acct
+        merchant_acct.remove_program_from_published_program_configuration(program.key_in_str)
+        
+        cls.__remove_schedule_program(program)
+        
     def to_program_configuration(self):
         program_configuration = {
                                 'merchant_acct_key'                 : self.parent_key,
@@ -163,6 +260,18 @@ class ProgramBase(BaseNModel, DictModel):
         
         return program_configuration
     
+
+class BasicRewardProgram(BaseProgram):
+    
+    @property
+    def completed_prograss_in_percentage(self):
+        return program_conf.program_completed_progress_percentage(self.completed_status)
+    
+    @property
+    def completed_status_index(self):
+        return program_conf.get_program_completed_status_index(self.completed_status)
+    
+        
     @classmethod
     def update_program_base_data(cls, program, reward_base=None, reward_format=None,  
                                  desc=None,start_date=None, end_date=None, modified_by=None):
@@ -190,7 +299,7 @@ class ProgramBase(BaseNModel, DictModel):
         program.end_date               = end_date
         program.modified_by            = modified_by.create_ndb_key()
         program.modified_by_username   = modified_by_username
-        program.completed_status       = program_conf.PROGRAM_STATUS_REWARD_BASE
+        program.completed_status       = program_conf.PROGRAM_STATUS_PROGRAM_BASE 
         program.program_settings       = program_settings
         
         
@@ -353,45 +462,6 @@ class ProgramBase(BaseNModel, DictModel):
         program.program_settings       = program_settings
         
         
-        program.put()
-        
-        return program
-    
-    @classmethod
-    def update_prorgram_membership_type_data(cls, program, is_multitier_membership=False, modified_by=None):
-        modified_by_username = None
-        
-        if is_not_empty(modified_by):
-            if isinstance(modified_by, MerchantUser):
-                modified_by_username = modified_by.username
-        
-        program_settings = program.program_settings
-        program_settings['is_multitier_membership'] = is_multitier_membership
-            
-        program.modified_by            = modified_by.create_ndb_key()
-        program.modified_by_username   = modified_by_username
-        program.program_settings       = program_settings
-        
-        program.put()
-        
-        return program
-    
-    @classmethod
-    def update_prorgram_exclusivity_data(cls, program, exclusivity_configuration={}, modified_by=None):
-        modified_by_username = None
-        
-        if is_not_empty(modified_by):
-            if isinstance(modified_by, MerchantUser):
-                modified_by_username = modified_by.username
-        
-        program_settings = program.program_settings
-        program_settings['exclusivity'] = exclusivity_configuration
-            
-        program.modified_by            = modified_by.create_ndb_key()
-        program.modified_by_username   = modified_by_username
-        program.completed_status       = program_conf.PROGRAM_STATUS_REWARD_EXCLUSIVITY
-        program.program_settings       = program_settings
-        
         program.put()
         
         return program
@@ -456,78 +526,14 @@ class ProgramBase(BaseNModel, DictModel):
         program.modified_by_username            = modified_by_username
         program.put()    
     
-    @classmethod
-    def __create_schedule_program(cls, merchant_acct, program):
-        if program.reward_base in (program_conf.SCHEDULE_BASED_PROGRAM):
-            if program.reward_base == program_conf.REWARD_BASE_ON_BIRTHDAY:
-                if program.giveaway_reward_when == program_conf.PROGRAM_SCHEDULE_TYPE_MONTH_START:
-                    MerchantScheduleProgram.create_first_day_of_month_schedule_program(merchant_acct, program)
-                elif program.giveaway_reward_when == program_conf.ADVANCE_IN_DAY:
-                    MerchantScheduleProgram.create_daily_schedule_program(merchant_acct, program)
-    
-    @classmethod
-    def __remove_schedule_program(cls, program):
-        if program.reward_base in (program_conf.SCHEDULE_BASED_PROGRAM):
-            MerchantScheduleProgram.remove_by_merchant_program(program)
-        
-    @classmethod
-    @model_transactional(desc="publish_program")
-    def publish_program(cls, program):
-        program.completed_status = program_conf.PROGRAM_STATUS_PUBLISH
-        program.published_datetime = datetime.now()
-        program.put()
-        
-        merchant_acct = program.merchant_acct
-        merchant_acct.update_published_program(program.to_program_configuration())
-        
-        cls.__create_schedule_program(merchant_acct, program)
-        
-    @classmethod
-    @model_transactional(desc="archive_program")
-    def archive_program(cls, program):
-        program.archived = True
-        program.archived_datetime = datetime.now()
-        program.put()
-        
-        merchant_acct = program.merchant_acct
-        merchant_acct.remove_program_from_published_program_configuration(program.key_in_str) 
-        
-        cls.__remove_schedule_program(program)
-        
-    
-    @classmethod
-    @model_transactional(desc="enable program")
-    def enable(cls, program):
-        program.enabled = True
-        program.put()
-        
-        if program.is_published:
-            merchant_acct = program.merchant_acct
-            merchant_acct.update_published_program(program.to_program_configuration())
-            
-            cls.__create_schedule_program(merchant_acct, program)
-            
-        
-    @classmethod
-    @model_transactional(desc="disable program")
-    def disable(cls, program):
-        program.enabled = False
-        program.put()  
-        
-        merchant_acct = program.merchant_acct
-        merchant_acct.remove_program_from_published_program_configuration(program.key_in_str)
-        
-        cls.__remove_schedule_program(program)
-    
-class MerchantProgram(ProgramBase):
-    #merchant_acct           = ndb.KeyProperty(name="merchant_acct", kind=MerchantAcct) 
     
     
+class MerchantProgram(BasicRewardProgram):
     dict_properties                     = ['reward_base', 'giveaway_method', 'reward_format', 'completed_status', 'start_date', 'end_date', 'desc', 'program_settings', 
                                            'created_datetime', 'modified_datetime',  'enabled','completed_status','is_enabled', 'is_disabled', 'is_review_state', 
                                            'is_published', 'archived', 'is_reward_amount_required', 'completed_prograss_in_percentage', 'completed_status_index',
                                            'exclusive_tags_list', 'exclusive_memberships_list', 'exclusive_tier_memberships_list',
-                                           'is_recurring_scheme', 'is_multitier_scheme', 'is_multitier_membership',
+                                           'is_recurring_scheme', 
                                            'expiration_type', 'expiration_date', 'expiration_value', 'is_expiration_date_type',
                                            'giveaway_reward_when', 'giveaway_reward_advance_in_day',
                                            'giveaway_system_condition', 'giveaway_system_condition_memberships_list', 'giveaway_system_condition_tier_memberships_list',
@@ -562,7 +568,7 @@ class MerchantProgram(ProgramBase):
                                                     'memberships'       : [],
                                                     'tier_memberships'  : [],
                                                     }
-        ''
+        
         program_settings = {
                             'exclusivity'               : exclusivity_configuration
                             }
@@ -578,7 +584,7 @@ class MerchantProgram(ProgramBase):
                                         end_date                            = end_date,
                                         created_by                          = created_by.create_ndb_key(),
                                         created_by_username                 = created_by_username,
-                                        completed_status                    = program_conf.PROGRAM_STATUS_REWARD_BASE,
+                                        completed_status                    = program_conf.PROGRAM_STATUS_PROGRAM_BASE,
                                         program_settings                    = program_settings,
                                         )
         
@@ -601,6 +607,310 @@ class MerchantProgram(ProgramBase):
     def list_archived_by_merchant_account(merchant_acct):
         return MerchantProgram.query(ndb.AND(MerchantProgram.archived==True), ancestor=merchant_acct.create_ndb_key()).fetch(limit=model_conf.MAX_FETCH_RECORD)
 
+class MerchantTierRewardProgram(MerchantProgram):
+    '''
+    Merchant acct as ancestor. Used for schedule program like birthday or push reward.
+    '''
+    
+    is_tier_recycle                     = ndb.BooleanProperty(required=True, default=True)
+    max_unlock_tier_count_per_trax      = ndb.IntegerProperty(required=False, default=999)
+    is_show_progress                    = ndb.BooleanProperty(required=True, default=True)
+    
+    dict_properties                     = ['completed_status', 'start_date', 'end_date', 'label', 'desc', 'program_settings', 'reward_format',
+                                           'created_datetime', 'modified_datetime',  'enabled','completed_status','is_enabled', 'is_disabled', 'is_review_state', 
+                                           'is_published', 'archived', 'completed_prograss_in_percentage', 'completed_status_index',
+                                           'exclusive_tags_list', 'exclusive_memberships_list', 'exclusive_tier_memberships_list',
+                                           'is_tier_recycle', 'is_show_progress', 'max_unlock_tier_count_per_trax',
+                                           'created_by_username', 'modified_by_username']
+    
+    @property
+    def completed_prograss_in_percentage(self):
+        return program_conf.tier_reward_program_completed_progress_percentage(self.completed_status)
+    
+    @property
+    def completed_status_index(self):
+        return program_conf.get_tier_reward_program_completed_status_index(self.completed_status)
+    
+    @property
+    def is_review_state(self):
+        return program_conf.is_existing_tier_reward_program_status_final_state(self.completed_status)
+    
+    @property
+    def program_tiers(self):
+        tier_settings_list = self.program_settings.get('tiers')
+        
+        logger.debug('tier_settings_list=%s', tier_settings_list)
+        
+        return sort_dict_list(tier_settings_list, 'unlock_tier_condition_value')
+        
+    
+    @property
+    def program_rewards(self):
+        tier_settings_list = self.program_settings.get('tiers') or []
+        reward_settings_list = []
+        for tier_setting in tier_settings_list:
+            reward_items = tier_setting.get('reward_items')
+            if reward_items:
+                for reward_setting in reward_items:
+                    reward_settings_list.append(reward_setting)
+        
+        return reward_settings_list
+    
+    @staticmethod
+    def create(merchant_acct, label=None, is_tier_recycle=True, is_show_progress=True, reward_format=None,
+               desc=None, start_date=None, end_date=None, created_by=None):
+        
+        created_by_username = None
+        if is_not_empty(created_by):
+            if isinstance(created_by, MerchantUser):
+                created_by_username = created_by.username
+        
+        exclusivity_configuration = {
+                                                    'tags'              : [],
+                                                    'memberships'       : [],
+                                                    'tier_memberships'  : [],
+                                                    }
+        
+        program_settings = {
+                            'exclusivity'               : exclusivity_configuration,
+                            'giveaway_methd'            : program_conf.PROGRAM_REWARD_GIVEAWAY_METHOD_SYSTEM,
+                            }
+        
+        
+        
+        tier_reward_program =  MerchantTierRewardProgram(
+                                        parent                              = merchant_acct.create_ndb_key(),
+                                        reward_base                         = program_conf.REWARD_BASE_ON_TIER,
+                                        reward_format                       = reward_format,
+                                        label                               = label,
+                                        desc                                = desc,
+                                        is_tier_recycle                     = is_tier_recycle,
+                                        is_show_progress                    = is_show_progress,
+                                        start_date                          = start_date,
+                                        end_date                            = end_date,
+                                        created_by                          = created_by.create_ndb_key(),
+                                        created_by_username                 = created_by_username,
+                                        completed_status                    = program_conf.PROGRAM_STATUS_PROGRAM_BASE,
+                                        program_settings                    = program_settings,
+                                        )
+        
+        tier_reward_program.put()
+        
+        return tier_reward_program
+    
+    @staticmethod
+    def update(program, label=None, is_tier_recycle=True, is_show_progress=True, reward_format=None,
+               desc=None, start_date=None, end_date=None, modified_by=None):
+        
+        if is_not_empty(modified_by):
+            if isinstance(modified_by, MerchantUser):
+                program.modified_by                          = modified_by.create_ndb_key()
+                program.modified_by_username                 = modified_by.username
+        
+        program.reward_format                       = reward_format
+        program.label                               = label
+        program.desc                                = desc
+        program.is_tier_recycle                     = is_tier_recycle
+        program.is_show_progress                    = is_show_progress
+        program.start_date                          = start_date
+        program.end_date                            = end_date
+        program.program_settings['giveaway_methd']  = program_conf.PROGRAM_REWARD_GIVEAWAY_METHOD_SYSTEM
+        program.put()
+        
+        return program
+    
+    @staticmethod
+    def remove_program_reward(program, reward_index, modified_by=None):
+        program_tier_settings_list  = program.program_settings['tiers']
+        if is_not_empty(modified_by):
+            if isinstance(modified_by, MerchantUser):
+                
+                program.modified_by             = modified_by.create_ndb_key()
+                program.modified_by_username    = modified_by.username
+        
+        if program_tier_settings_list:
+            latest_tier_settings_list = []
+            
+            for tier_setting in program_tier_settings_list:
+                reward_settings_list        = tier_setting.get('reward_items')
+                if reward_settings_list:
+                    latest_reward_setting_list  = []
+                    
+                    for reward_setting in reward_settings_list:
+                        if reward_setting.get('reward_index') != reward_index:
+                            latest_reward_setting_list.append(reward_setting)
+                    
+                    tier_setting['reward_items'] = latest_reward_setting_list
+                
+                
+                latest_tier_settings_list.append(tier_setting)
+        
+        program.program_settings['tiers'] =  latest_tier_settings_list   
+        program.put()
+    
+    @staticmethod
+    def add_program_reward(program, tier_index=None, merchant_voucher_key=None, voucher_amount=1, use_online=True, use_in_store=True, 
+                           effective_type=None, effective_value=None, effective_date=None,
+                           expiration_type=None, expiration_value=None, expiration_date=None,
+                           modified_by=None
+                           ):
+        program_tier_settings_list  = program.program_settings['tiers']
+        reward_index                = random_string(12)
+        if is_not_empty(modified_by):
+            if isinstance(modified_by, MerchantUser):
+                
+                program.modified_by             = modified_by.create_ndb_key()
+                program.modified_by_username    = modified_by.username
+        
+        if program_tier_settings_list:
+            latest_tier_settings_list = []
+            
+            for tier_setting in program_tier_settings_list:
+                if tier_setting.get('tier_index') == tier_index:
+                    reward_settings_list = tier_setting.get('reward_items')
+                    if not reward_settings_list:
+                        reward_settings_list = []
+                    
+                    reward_settings_list.append({
+                                                'tier_index'        : tier_index,
+                                                'reward_index'      : reward_index,
+                                                'voucher_key'       : merchant_voucher_key,
+                                                'voucher_amount'    : voucher_amount,
+                                                'use_online'        : use_online,
+                                                'use_in_store'      : use_in_store,
+                                                'effective_type'    : effective_type,
+                                                'effective_value'   : effective_value,
+                                                'effective_date'    : effective_date,
+                                                'expiration_type'   : expiration_type,
+                                                'expiration_value'  : expiration_value,
+                                                'expiration_date'   : expiration_date,
+                                                })
+            
+                    tier_setting['reward_items'] = reward_settings_list
+                    
+                latest_tier_settings_list.append(tier_setting)
+                
+                    
+        #program.completed_status           = program_conf.PROGRAM_STATUS_DEFINE_REWARD
+        program.program_settings['tiers'] =  latest_tier_settings_list   
+        program.put()
+        
+        
+        
+        
+    @staticmethod
+    def add_program_tier(program, tier_label=None, unlock_tier_message=None, unlock_tier_condition=None, unlock_tier_condition_value=None, modified_by=None):
+        tier_index = random_string(12)
+        tier_settings = {
+                        'tier_index'                    : tier_index,
+                        'tier_label'                    : tier_label,
+                        'unlock_tier_message'           : unlock_tier_message,
+                        'unlock_tier_condition'         : unlock_tier_condition,
+                        'unlock_tier_condition_value'   : unlock_tier_condition_value,
+                        }
+        
+        program_settings = program.program_settings
+        
+        tier_list = program_settings.get('tiers')
+        if tier_list:
+            tier_list.append(tier_settings)
+        else:
+            tier_list = [tier_settings]
+            
+        modified_by_username = None
+        if is_not_empty(modified_by):
+            if isinstance(modified_by, MerchantUser):
+                modified_by_username = modified_by.username    
+            
+        program_settings['tiers']       = tier_list
+        program.program_settings        = program_settings
+        
+        if modified_by:
+            program.modified_by             = modified_by.create_ndb_key()
+            program.modified_by_username    = modified_by_username
+            
+        program.put()
+        
+    @staticmethod
+    def update_program_tier_settings(program, program_tier_settings_list, modified_by=None):
+        modified_by_username = None
+        if is_not_empty(modified_by):
+            if isinstance(modified_by, MerchantUser):
+                modified_by_username = modified_by.username    
+        
+        existing_program_tier_settings_list = program.program_settings.get('tiers') or []
+        
+        for new_tier_settings in program_tier_settings_list:
+            logger.debug('new_tier_settings=%s', new_tier_settings);
+            for existing_tier_setting in existing_program_tier_settings_list:
+                logger.debug('existing_tier_setting=%s', existing_tier_setting);
+                if existing_tier_setting.get('reward_items'):
+                    if existing_tier_setting.get('tier_index') == new_tier_settings.get('tier_index'):
+                        new_tier_settings['reward_items'] = existing_tier_setting.get('reward_items')
+                        break
+                else:
+                    continue
+                     
+        
+        program.program_settings['tiers']  = program_tier_settings_list
+        program.completed_status           = program_conf.PROGRAM_STATUS_DEFINE_TIER
+        if modified_by:
+            program.modified_by             = modified_by.create_ndb_key()
+            program.modified_by_username    = modified_by_username
+            
+        program.put() 
+    
+    @staticmethod    
+    def update_program_reward_settings(program, modified_by=None):
+        modified_by_username = None
+        if is_not_empty(modified_by):
+            if isinstance(modified_by, MerchantUser):
+                modified_by_username = modified_by.username    
+        
+        program.completed_status           = program_conf.PROGRAM_STATUS_DEFINE_REWARD
+        if modified_by:
+            program.modified_by             = modified_by.create_ndb_key()
+            program.modified_by_username    = modified_by_username
+            
+        program.put()        
+    
+    @staticmethod
+    def remove_tier(program, tier_index, modified_by=None):
+        
+        program_settings = program.program_settings
+        
+        tier_list = program_settings.get('tiers')
+        if tier_list:
+            index_no = 0
+            for tier_setting in tier_list:
+                if tier_setting.get('tier_index') == tier_index:
+                    del tier_list[index_no]
+                    break
+            
+                index_no+=1
+        
+        modified_by_username = None
+        if is_not_empty(modified_by):
+            if isinstance(modified_by, MerchantUser):
+                modified_by_username = modified_by.username
+                
+        program_settings['tiers']       = tier_list
+        program.program_settings        = program_settings
+        
+        if modified_by:
+            program.modified_by             = modified_by.create_ndb_key()
+            program.modified_by_username    = modified_by_username
+            
+        program.put()
+    
+    @staticmethod
+    def list_by_merchant_acct(merchant_acct):
+        result =  MerchantTierRewardProgram.query(ancestor=merchant_acct.create_ndb_key()).fetch(limit=conf.MAX_FETCH_RECORD)
+        if result:
+            return sort_list(result, 'created_datetime', reverse_order=True)
+        else:
+            return []
+    
 class MerchantScheduleProgram(BaseNModel, DictModel):
     '''
     Merchant acct as ancestor. Used for schedule program like birthday or push reward.
