@@ -159,7 +159,7 @@ class CustomerRedemption(BaseNModel, DictModel):
         return CustomerRedemption.count_with_condition_query(query, limit=limit)
     
     @staticmethod
-    def create(customer, reward_format, redeemed_amount, redeemed_outlet, 
+    def create(customer, reward_format, redeemed_amount, redeemed_outlet, transaction_id=None, 
                redeemed_voucher_keys_list=None, invoice_id=None, remarks=None, redeemed_by=None, redeemed_datetime=None):
         
         reward_summary              = customer.reward_summary
@@ -170,7 +170,7 @@ class CustomerRedemption(BaseNModel, DictModel):
                 redeemed_by_username = redeemed_by.username
 
         
-        redeem_transaction_id = generate_transaction_id(prefix='r')
+        redeem_transaction_id = transaction_id or generate_transaction_id(prefix='r')
         
         if redeemed_datetime is None:
             redeemed_datetime = datetime.utcnow()
@@ -250,8 +250,11 @@ class CustomerRedemption(BaseNModel, DictModel):
                                 
                                 }
                 
-            
-            reward_summary[reward_format]['amount'] -= redeemed_amount
+            reward_balance = reward_summary[reward_format]['amount'] - redeemed_amount
+            if reward_balance<0:
+                reward_balance = 0
+                
+            reward_summary[reward_format]['amount'] = reward_balance
                                 
             
         elif reward_format == program_conf.REWARD_FORMAT_VOUCHER:
