@@ -7,7 +7,8 @@ from google.cloud import ndb
 from trexmodel.models.datastore.ndb_models import BaseNModel, DictModel
 from trexmodel.models.datastore.user_models import User
 from trexmodel.models.datastore.merchant_models import MerchantAcct, Outlet, MerchantUser
-from trexmodel.models.datastore.program_models import MerchantProgram
+from trexmodel.models.datastore.program_models import MerchantProgram,\
+    MerchantTierRewardProgram
 from trexmodel.models.datastore.voucher_models import MerchantVoucher
 from trexlib.utils.string_util import is_empty, is_not_empty
 import logging
@@ -17,6 +18,8 @@ from datetime import datetime, timedelta
 from trexmodel.utils.model.model_util import generate_transaction_id,\
     string_to_key_property
 from six import string_types
+from trexmodel.models.datastore.customer_models import Customer
+from flask_babel import gettext
 
 logger = logging.getLogger('model')
 
@@ -67,6 +70,10 @@ class CustomerEntitledReward(RewardEntitlement):
     def entitled_customer_key(self):
         return self.key.parent().urlsafe().decode('utf-8')
     
+    @property
+    def reward_program_key(self):
+        return self.reward_program.urlsafe().decode('utf-8')
+    
     @property
     def rewarded_by_merchant_acct_entity(self):
         return MerchantAcct.fetch(self.merchant_acct.urlsafe())
@@ -115,6 +122,10 @@ class CustomerCountableReward(CustomerEntitledReward):
     reward_amount               = ndb.FloatProperty(required=True, default=0)
     used_reward_amount          = ndb.FloatProperty(required=True, default=0)
     
+    @property
+    def is_reach_reward_limit(self):
+        return self.status == program_conf.REWARD_STATUS_REACH_LIMIT
+    
     @property
     def reward_balance(self):
         return self.reward_amount - self.used_reward_amount
@@ -144,7 +155,10 @@ class CustomerCountableReward(CustomerEntitledReward):
     
     @property
     def reward_brief(self):
-        return 'Entitle {reward_amount} {reward_format}'.format(reward_amount=self.reward_amount, reward_format=self.reward_format_label)
+        if self.is_reach_reward_limit:
+            return gettext('Not entitle any reward due to reach rewward limit')
+        else:
+            return gettext('Entitle {reward_amount} {reward_format}').format(reward_amount=self.reward_amount, reward_format=self.reward_format_label)
     
     def to_reward_summary(self):
         return {
@@ -152,6 +166,8 @@ class CustomerCountableReward(CustomerEntitledReward):
                 'amount'                : self.reward_amount,
                 'used_amount'           : self.used_reward_amount,
                 'expiry_date'           : self.expiry_date.strftime('%d-%m-%Y'),
+                'program_key'           : self.reward_program_key,
+                'is_reach_reward_limit' : self.is_reach_reward_limit,
                 }
     
     @classmethod
@@ -159,7 +175,7 @@ class CustomerCountableReward(CustomerEntitledReward):
                effective_date=None, expiry_date=None, 
                transaction_id=None, invoice_id=None,
                rewarded_by=None, program_key=None,
-               rewarded_datetime = None
+               rewarded_datetime = None, status=program_conf.REWARD_STATUS_VALID
                ):
         
         if is_empty(transaction_id):
@@ -189,6 +205,8 @@ class CustomerCountableReward(CustomerEntitledReward):
                                             reward_program          = reward_program,
                                             rewarded_datetime       = rewarded_datetime,
                                             
+                                            status                  = status,
+                                            
                                             )
         
         customer_reward.put()
@@ -254,7 +272,6 @@ class RevertedCustomerStampReward(RevertedCustomerCountableReward):
 class CustomerEntitledVoucher(CustomerEntitledReward):
     
     entitled_voucher            = ndb.KeyProperty(name='entitled_voucher', kind=MerchantVoucher)
-    
     redeem_code                 = ndb.StringProperty(required=False)
     redeemed_datetime           = ndb.DateTimeProperty(required=False)
     
@@ -312,7 +329,7 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
         return MerchantVoucher.fetch(self.entitled_voucher.urlsafe())
     
     @property
-    def merchant_voucher(self):
+    def merchant_voucher_entity(self):
         return MerchantVoucher.fetch(self.entitled_voucher.urlsafe())
     
     @property
@@ -327,7 +344,7 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
                 }
     
     @staticmethod
-    def create(entitled_voucher, customer_acct, transact_outlet, 
+    def create(merchant_voucher, customer_acct, transact_outlet=None, 
                effective_date=None, expiry_date=None,
                transaction_id=None, invoice_id=None,
                rewarded_by=None, rewarded_datetime=None):
@@ -339,7 +356,7 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
         
         customer_entiteld_voucher = CustomerEntitledVoucher(
                                             parent                  = customer_acct.create_ndb_key(),
-                                            entitled_voucher        = entitled_voucher.create_ndb_key(),
+                                            entitled_voucher        = merchant_voucher.create_ndb_key(),
                                             
                                             effective_date          = effective_date,
                                             expiry_date             = expiry_date,
@@ -384,17 +401,196 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
         if customers_vouchers_list:
             
             for customer_voucher in customers_vouchers_list:
-                voucher         = customer_voucher.entitled_voucher
-                voucher_key      = voucher.urlsafe().decode('utf-8')
-                voucher_count    = entitled_voucher_summary.get(voucher_key)
-                    
-                if voucher_count:
-                    entitled_voucher_summary[voucher_key] = voucher_count + 1
+                merchant_voucher        = customer_voucher.merchant_voucher_entity
+                merchant_voucher_key    = customer_voucher.merchant_voucher_key
+                voucher_summary         = entitled_voucher_summary.get(merchant_voucher_key)
+                new_redeem_info         = {
+                                            'redeem_code'       : customer_voucher.redeem_code,
+                                            'effective_date'    : customer_voucher.effective_date.strftime('%d-%m-%Y'),
+                                            'expiry_date'       : customer_voucher.expiry_date.strftime('%d-%m-%Y'),
+                                            }    
+                if voucher_summary:
+                    entitled_voucher_summary[merchant_voucher_key]['redeem_info_list'].append(new_redeem_info)
                 else:
-                    entitled_voucher_summary[voucher_key] = 1
+                    entitled_voucher_summary[merchant_voucher_key]={
+                                                                    'label'             : merchant_voucher.label,
+                                                                    'image_url'         : merchant_voucher.image_public_url,
+                                                                    'redeem_info_list'  : [new_redeem_info]
+                                                                    }
         
         customer.entitled_voucher_summary = entitled_voucher_summary
         customer.put()
+
+class CustomerEntitledTierRewardSummary(BaseNModel, DictModel):
+    '''
+    Customer as ancestor
+    
+    tier_summary consist of list of tier_details details
+    
+    where, tier_details is like below
+    
+    tier_details = {
+                            'tier_index'                : ...,
+                            'tier_label'                : ...,
+                            'unlock_tier_message'       : ...,
+                            'unlock_status'             : ...,
+                            'unlock_condition'          : ...,
+                            'unlock_condition_value'    : ..., 
+                            }
+    
+    
+    '''
+    tier_reward_program     = ndb.KeyProperty(name='tier_reward_program', kind=MerchantTierRewardProgram)
+    cycle_start_datetime    = ndb.DateTimeProperty(required=True, auto_now_add=True)
+    cycle_end_datetime      = ndb.DateTimeProperty(required=False)
+    program_end_date        = ndb.DateProperty(required=True)
+    tier_summary            = ndb.JsonProperty()
+    is_valid                = ndb.BooleanProperty(required=True, default=True)
+    allow_recycle           = ndb.BooleanProperty(required=True, default=True)
+    modified_datetime       = ndb.DateTimeProperty(required=True, auto_now=True)
+    
+    dict_properties     = ['cycle_start_datetime', 'cycle_end_datetime', 'is_valid', 'program_end_date', 'tier_summary']
+    
+    @property
+    def cycle_completed_datetime(self):
+        return self.cycle_end_datetime
+    
+    @property
+    def tier_reward_program_entity(self):
+        return MerchantTierRewardProgram.fetch(self.tier_reward_program.urlsafe())
+    
+    @property
+    def is_cycle_completed(self):
+        is_completed = True
+        if self.tier_summary:
+            for tier_details in self.tier_summary.get('tiers'):
+                if tier_details.get('unlock_status') == False:
+                    is_completed = False
+                    break
+        return is_completed
+    
+    def to_program_tier_status_summary(self):
+        tier_details_list           = self.tier_summary.get('tiers')
+        tier_reward_program         = self.tier_reward_program_entity
+        
+        last_completed_unlock_condition_value   = 0
+        found_incomplete_tier                   = False
+        
+        for tier_details in tier_details_list:
+            if tier_details.get('unlock_status'):
+                last_completed_unlock_condition_value   = int(tier_details.get('unlock_condition_value'))
+                tier_details['completed_percentage']    = 100 
+            else:
+                tier_unlock_value           = int(tier_details.get('unlock_value')) or 0
+                tier_unlock_condition_value = int(tier_details.get('unlock_condition_value'))
+                if found_incomplete_tier:
+                    tier_completed_percentage = '0'
+                else:
+                    if tier_unlock_value>0:
+                        tier_completed_percentage = '%.2f' % (100*(tier_unlock_value-last_completed_unlock_condition_value)/(tier_unlock_condition_value-last_completed_unlock_condition_value))
+                    else:
+                        tier_completed_percentage = '%.2f' % (100*tier_unlock_value/tier_unlock_condition_value)
+                
+                tier_details['completed_percentage'] = tier_completed_percentage
+                
+                found_incomplete_tier = True
+                
+        return {
+                'label' : tier_reward_program.label,
+                'tiers' :tier_details_list
+                }
+        
+    @staticmethod
+    def list_tier_reward_summary_by_customer(customer_acct):
+        today = datetime.utcnow().date()
+        return CustomerEntitledTierRewardSummary.query(ndb.AND(
+                                                            CustomerEntitledTierRewardSummary.is_valid==True, 
+                                                            CustomerEntitledTierRewardSummary.program_end_date>=today
+                                                            ),
+                                                        ancestor = customer_acct.create_ndb_key()
+                                                    ).fetch(limit=conf.MAX_FETCH_RECORD)
+    
+    @staticmethod
+    def get_customer_tier_reward_program_summary(customer_acct, tier_reward_program):
+        tier_reward_summary         = CustomerEntitledTierRewardSummary.query(
+                                                                    ndb.AND(
+                                                                        CustomerEntitledTierRewardSummary.tier_reward_program==tier_reward_program.create_ndb_key() 
+                                                                        ),
+                                                                    ancestor=customer_acct.create_ndb_key()).get()
+    
+        return tier_reward_summary
+    
+    @staticmethod
+    def create(customer_acct, tier_reward_program, cycle_start_datetime=None):
+        
+        if cycle_start_datetime is None:
+            cycle_start_datetime = datetime.utcnow()
+        
+        tier_setting_list = tier_reward_program.program_tiers
+        tier_reward_summary_list = []
+        for tier_setting in tier_setting_list:
+            tier_summary = {
+                            'tier_index'                : tier_setting.get('tier_index'),
+                            'tier_label'                : tier_setting.get('tier_label'),
+                            'unlock_tier_message'       : tier_setting.get('unlock_tier_message'),
+                            'unlock_status'             : False,
+                            'unlock_condition'          : tier_setting.get('unlock_tier_condition'),
+                            'unlock_condition_value'    : tier_setting.get('unlock_tier_condition_value'), 
+                            'unlock_value'              : .0,
+                            }  
+            tier_reward_summary_list.append(tier_summary)  
+            
+        
+        reward_summary = CustomerEntitledTierRewardSummary(
+                                            parent                  = customer_acct.create_ndb_key(),
+                                            tier_reward_program     = tier_reward_program.create_ndb_key(),
+                                            program_end_date        = tier_reward_program.end_date,
+                                            cycle_start_datetime    = cycle_start_datetime,
+                                            tier_summary            = {'tiers': tier_reward_summary_list},
+                                            allow_recycle           = tier_reward_program.is_tier_recycle,
+                                            )
+        
+        reward_summary.put()
+        
+        return reward_summary
+    
+    @staticmethod
+    def update(customer_tier_reward_summary, tier_summary=None, cycle_start_datetime=None, cycle_end_datetime=None):
+        if cycle_start_datetime:
+            customer_tier_reward_summary.cycle_start_datetime = cycle_start_datetime
+        
+        if cycle_end_datetime:
+            customer_tier_reward_summary.cycle_end_datetime = cycle_end_datetime
+            
+        customer_tier_reward_summary.tier_summary = tier_summary
+        if customer_tier_reward_summary.is_cycle_completed:
+            customer_tier_reward_summary.cycle_end_datetime = datetime.utcnow()
+        
+        customer_tier_reward_summary.put()
+        
+    @staticmethod
+    def restart_cycle(customer_tier_reward_summary, cycle_start_datetime=None):
+        if cycle_start_datetime is None:
+            customer_tier_reward_summary.cycle_start_datetime   = datetime.utcnow()
+        else:
+            customer_tier_reward_summary.cycle_start_datetime   = cycle_start_datetime
+            
+        customer_tier_reward_summary.cycle_end_datetime     = None
+        
+        tier_details_list = customer_tier_reward_summary.tier_summary.get('tiers')
+        
+        for tier_details in tier_details_list:
+            tier_details['unlock_status']      = False
+            tier_details['unlock_datetime']    = None
+            tier_details['unlock_value']       = .0
+        
+        logger.debug('tier_details_list=%s', tier_details_list)
+            
+        customer_tier_reward_summary.tier_summary['ties'] = tier_details_list
+        
+        customer_tier_reward_summary.put()    
+        
+        
     
 class VoucherRewardDetailsForUpstreamData(object):    
     '''
