@@ -16,12 +16,13 @@ import logging
 from trexmodel import conf, program_conf
 from datetime import datetime,date, timedelta
 from trexmodel.models.datastore.reward_models import CustomerPointReward,\
-    CustomerStampReward, CustomerEntitledVoucher
+    CustomerStampReward, CustomerEntitledVoucher, CustomerEntitledReward
 from trexmodel.models.datastore.customer_model_helpers import update_reward_summary_with_reverted_reward,\
     update_customer_entiteld_voucher_summary_after_reverted_voucher
 from trexmodel.utils.model.model_util import generate_transaction_id
 from gettext import gettext
 from trexmodel.models.datastore.prepaid_models import CustomerPrepaidReward
+from trexmodel.models.datastore.program_models import MerchantTierRewardProgram
 
 logger = logging.getLogger('model')
 
@@ -36,6 +37,8 @@ class CustomerTransaction(BaseNModel, DictModel):
     transact_datetime           = ndb.DateTimeProperty(required=True)
     created_datetime            = ndb.DateTimeProperty(required=True, auto_now=True)
     
+    transact_timestamp          = ndb.FloatProperty(required=False)
+    
     transaction_id              = ndb.StringProperty(required=True)
     invoice_id                  = ndb.StringProperty(required=False)
     remarks                     = ndb.StringProperty(required=False)
@@ -51,6 +54,7 @@ class CustomerTransaction(BaseNModel, DictModel):
     entitled_reward_summary     = ndb.JsonProperty()
     entitled_voucher_summary    = ndb.JsonProperty()
     entitled_prepaid_summary    = ndb.JsonProperty()
+    entitled_program_summary    = ndb.JsonProperty()
     
     reward_giveaway_method      = ndb.StringProperty(required=False, default=program_conf.PROGRAM_REWARD_GIVEAWAY_METHOD_SYSTEM)
     
@@ -109,6 +113,13 @@ class CustomerTransaction(BaseNModel, DictModel):
     def transact_by_user(self):
         if self.transact_by:
             return MerchantUser.fetch(self.transact_by.urlsafe())
+        
+    @property
+    def after_deduct_tax_sales_amount(self):
+        if self.tax_amount:
+            return self.transact_amount - self.tax_amount
+        else:
+            return self.transact_amount
     
     @staticmethod
     def update_transaction_reward_have_been_redeemed(transaction_id):
@@ -283,6 +294,20 @@ class CustomerTransaction(BaseNModel, DictModel):
         
         return CustomerTransaction.list_all_with_condition_query(query, offset=offset, limit=limit, start_cursor=start_cursor, return_with_cursor=return_with_cursor)
     
+    @staticmethod
+    def list_customer_transaction_by_transact_timestamp(customer_acct, transact_timestamp_from=None, transact_timestamp_to=None):
+        return CustomerTransaction.query(ndb.AND(
+                                                    CustomerTransaction.transact_timestamp>transact_timestamp_from,
+                                                    CustomerTransaction.transact_timestamp<=transact_timestamp_to
+                                                  ),ancestor = customer_acct.create_ndb_key()).fetch(limit=conf.MAX_FETCH_RECORD)
+        
+    @staticmethod
+    def list_customer_transaction_by_transact_datetime(customer_acct, transact_datetime_from=None, transact_datetime_to=None):
+        return CustomerTransaction.query(ndb.AND(
+                                                    CustomerTransaction.transact_datetime>=transact_datetime_from,
+                                                    CustomerTransaction.transact_datetime<transact_datetime_to
+                                                  ),ancestor = customer_acct.create_ndb_key()).fetch(limit=conf.MAX_FETCH_RECORD)    
+    
     @staticmethod
     def get_by_transaction_id(transaction_id):
         return CustomerTransaction.query(CustomerTransaction.transaction_id==transaction_id).get()
@@ -379,6 +404,8 @@ class CustomerTransaction(BaseNModel, DictModel):
                                     ))
         
         return CustomerTransaction.count_with_condition_query(query, limit=limit)
+
+
     
 class CustomerTransactionWithRewardDetails(object):    
     
