@@ -6,7 +6,7 @@ Created on 10 Apr 2020
 from google.cloud import ndb
 from trexmodel.models.datastore.ndb_models import BaseNModel, DictModel
 #from trexlib.utils.common.cache_util import cache
-from trexlib.utils.string_util import random_number, is_not_empty
+from trexlib.utils.string_util import random_number, is_not_empty, random_string
 from flask_login import UserMixin
 import logging  
 from trexlib.utils.security_util import generate_user_id, hash_password
@@ -17,6 +17,8 @@ import json
 from json import JSONEncoder
 from flask_babel import gettext
 from trexlib.utils.common.date_util import to_day_of_year
+from datetime import datetime, timedelta
+
 
 logger = logging.getLogger("model")
 
@@ -115,10 +117,24 @@ class UserBase(UserMin):
     #---------------------------------------------------------------------------
     # User account activation required fields
     #---------------------------------------------------------------------------
-    is_email_verified           = ndb.BooleanProperty(required=False, default=False)
-    is_mobile_phone_verified    = ndb.BooleanProperty(required=False, default=False)
-
-    modified_datetime           = ndb.DateTimeProperty(required=True, auto_now=True)
+    is_email_verified                   = ndb.BooleanProperty(required=False, default=False)
+    is_mobile_phone_verified            = ndb.BooleanProperty(required=False, default=False)
+    
+    email_verified_datetime             = ndb.DateTimeProperty(required=False)
+    mobile_phone_verified_datetime      = ndb.DateTimeProperty(required=False)
+    
+    email_vc                            = ndb.StringProperty(required=False)
+    email_vc_expiry_datetime            = ndb.DateTimeProperty(required=False)
+    
+    mobile_phone_vc                     = ndb.StringProperty(required=False)
+    mobile_phone_vc_expiry_datetime     = ndb.DateTimeProperty(required=False)
+    
+    modified_datetime                   = ndb.DateTimeProperty(required=True, auto_now=True)
+    
+    request_reset_password_datetime             = ndb.DateTimeProperty(required=False)
+    request_reset_password_expiry_datetime      = ndb.DateTimeProperty(required=False)
+    request_reset_password_token                = ndb.StringProperty(required=False)
+    reset_password_updated_datetime             = ndb.DateTimeProperty(required=False)
     
     @property
     def age(self):
@@ -131,6 +147,10 @@ class UserBase(UserMin):
         else:
             return 0
     
+    @classmethod
+    def get_by_reference_code(cls, reference_code):
+        return User.query(ndb.AND(User.reference_code==reference_code)).get()
+    
     @classmethod
     def get_by_email(cls, email):
         return User.query(ndb.AND(User.email==email)).get()
@@ -139,10 +159,14 @@ class UserBase(UserMin):
     def get_by_mobile_phone(cls, mobile_phone):
         return User.query(ndb.AND(User.mobile_phone==mobile_phone)).get()
     
+    @classmethod
+    def get_by_reset_password_token(cls, request_reset_password_token):
+        return User.query(ndb.AND(User.request_reset_password_token==request_reset_password_token)).get()
+    
     @classmethod
     def create(cls, name=None, email=None, mobile_phone=None, 
                gender=None, birth_date=None, 
-               password=None):
+               password=None, create_email_vc=False, create_mobile_phone_vc=False):
         
         user_id             = generate_user_id()
         reference_code      = random_number(16) 
@@ -163,6 +187,14 @@ class UserBase(UserMin):
         
         hashed_password = hash_password(user_id, password)
         created_user.password = hashed_password
+        
+        if create_email_vc:
+            created_user.email_vc                               = random_number(6)
+            created_user.email_vc_expiry_datetime               = datetime.now() + timedelta(minutes=10)
+        
+        if create_mobile_phone_vc:
+            created_user.mobile_phone_vc                        = random_number(6)
+            created_user.mobile_phone_vc_expiry_datetime        = datetime.now() + timedelta(minutes=10)   
             
         created_user.put()
         
@@ -205,7 +237,7 @@ class UserBase(UserMin):
     def update_biodata(cls, user_acct, gender=None, birth_date=None):
         user_acct.gender        = gender
         user_acct.birth_date    = birth_date
-        birth_day_in_year   = 0
+        birth_day_in_year       = 0
         
         if birth_date:
             birth_date_date_str = birth_date.strftime('%d/%m')
@@ -214,7 +246,40 @@ class UserBase(UserMin):
         user_acct.birth_date_date_str   = birth_date_date_str
         user_acct.birth_day_in_year     = birth_day_in_year
         
-        user_acct.put()    
+        user_acct.put() 
+    
+    def reset_password_request(self):
+        self.request_reset_password_token = random_string(16)
+        self.request_reset_password_datetime = datetime.utcnow()
+        self.request_reset_password_expiry_datetime = self.request_reset_password_datetime + timedelta(minutes=5)
+        self.put()
+        
+    def reset_password(self, new_password):
+        hashed_password = hash_password(self.user_id, new_password)
+        self.password   = hashed_password
+        
+        self.reset_password_updated_datetime = datetime.utcnow()
+        self.put()    
+        
+    def mark_email_verified(self):
+        self.is_email_verified = True
+        self.email_verified_datetime = datetime.now()
+        self.put()
+        
+    def mark_mobile_phone_verified(self):
+        self.is_mobile_phone_verified = True
+        self.mobile_phone_verified_datetime = datetime.utcnow()
+        self.put()    
+    
+    def reset_email_vc(self):
+        self.email_vc = random_number(6)
+        self.email_vc_expiry_datetime = datetime.utcnow() + timedelta(minutes=10)
+        self.put() 
+        
+    def reset_mobile_phone_vc(self):
+        self.email_vc = random_number(6)
+        self.mobile_phone_vc_expiry_datetime = datetime.utcnow() + timedelta(minutes=10)  
+        self.put()   
         
 class User(UserBase):
     redeem_pin               = ndb.StringProperty(required=False)
